from tkinter import *
from time import sleep


class HanoiTower(Canvas):

    def __init__(self):
        super().__init__(bg='grey40')
        self.n_game = None
        self.text_info = None
        self.previous_column = 0
        self._from = None
        self._to = None
        self.end_position = None
        self.end_position_rect = None
        self.choose_button = None
        self.choose_text = None
        self.selected = False
        self.master.title('Hanoi Tower')
        self.pack(fill=BOTH, expand=TRUE)
        self.update()
        self.width = self.winfo_width()
        self.height = self.winfo_height()
        self.bagels = [[], [], []]
        self.min_bagel = 10
        self.draw_towers()
        self.choosing_start_position()
        self.colors = ('grey20', 'grey30', 'grey40', 'purple', 'blue', 'cyan', 'green', 'yellow', 'orange', 'red')

    def draw_towers(self):
        radius = (self.width / 60)
        height = (self.height * 0.7)
        base_radius = (self.width / 7)
        base_height = (self.height / 30)
        for i in range(3):
            down_point = (self.width * (i / 3 + 1 / 6), self.height * 0.85)
            self.create_rectangle(down_point[0] - radius,
                                  down_point[1] - height,
                                  down_point[0] + radius,
                                  down_point[1],
                                  fill='white', outline='black')
            self.create_rectangle(down_point[0] - base_radius,
                                  down_point[1],
                                  down_point[0] + base_radius,
                                  down_point[1] + base_height,
                                  fill='white', outline='black')
        self.text_info = self.create_text(self.width / 2, self.height / 2, anchor=CENTER,
                                          font=f'Arial {self.height//8} bold')
        self.update()

    def choosing_start_position(self):
        # print('Choosing start position')
        self.choose_text = self.create_text(self.width / 2, self.height / 20,
                                            font=f'Arial {self.height // 40} bold',
                                            anchor=CENTER,
                                            text='Choose start position and press "OK"')
        self.choose_button = Button(self, anchor=CENTER, background='cyan',
                                    font=f'Arial {self.height // 40} bold',
                                    text='OK', padx=70)
        self.choose_button.place(x=self.width / 2, y=self.height * 19 / 20, anchor=CENTER)
        self.bind('<Button-1>', self.add_bagel)
        self.n_game = Button(self, anchor=CENTER, background='cyan',
                             font=f'Arial {self.height // 40} bold',
                             text='New game', command=self.new_game,
                             padx=40)
        self.n_game.place(x=self.width * 5 // 6, y=self.height * 19 / 20, anchor=CENTER)

    def new_game(self):
        self.destroy()
        HanoiTower()

    def add_bagel(self, event):
        self.choose_button.configure(command=self.choosing_end_position)
        if self.min_bagel < 1:
            return
        i = event.x // (self.width // 3)
        bagel = self.draw_bagel(bagel_column=i, column_amount=len(self.bagels[i]), bagel_size=self.min_bagel)
        self.bagels[i].append([self.min_bagel, bagel, i])
        self.previous_column = i
        self.min_bagel -= 1

    def choosing_end_position(self):
        # print('Start position selected. Choosing end position')
        # print(self.bagels)
        self.itemconfigure(self.choose_text, text='Choose end position and press "OK"')
        self.update()
        self.unbind('<Button-1>')
        self.bind('<Button-1>', self.draw_end_position)

    def automatically(self):
        # print('automatically')
        self.delete(self.choose_text)
        self.choose_button.destroy()
        self.unbind('<Button-1>')
        for n in range(1, 11-self.min_bagel):
            maximum = self.min_bagel + n
            max_column = 0
            for i, column in enumerate(self.bagels):
                for bagel in column:
                    if bagel[0] == maximum:
                        max_column = i
            if max_column == self.previous_column:
                continue
            temp_column = 3 - max_column - self.previous_column
            if ((self.bagels[temp_column] and self.bagels[temp_column][-1][0] - self.bagels[max_column][-1][0] == 1)
                    or (n == 10 and not self.bagels[temp_column]) and temp_column == self.end_position):
                self._from = max_column
                self._to = temp_column
                # print('Moving without recursion')
                self.move_bagel()
                sleep(0.05)
                max_column = temp_column
            self.auto_moving(fr=self.previous_column, to=max_column, n=n - 1)
            self.previous_column = max_column
        if self.previous_column != self.end_position:
            self.auto_moving(fr=self.previous_column, to=self.end_position, n=10 - self.min_bagel)

    def auto_moving(self, fr, to, n):
        temp = 3 - fr - to
        if n <= 1:
            self._from = fr
            self._to = to
            self.move_bagel()
            return
        self.auto_moving(fr=fr, to=temp, n=n - 1)
        sleep(0.05)
        self._from = fr
        self._to = to
        self.move_bagel()
        sleep(0.05)
        self.auto_moving(fr=temp, to=to, n=n - 1)
        sleep(0.05)

    def gaming(self):
        # print(f"End position - {self.end_position}. It is gaming")
        self.itemconfigure(self.choose_text, text='Move rings from one tower to another. Try to solve this')
        self.unbind('<Button-1>')
        self.bind('<Button-1>', self.get_from_to)
        self.choose_button.configure(command=self.automatically, text='Auto')

    def get_from_to(self, event):
        i = event.x // (self.width // 3)
        if self._from is None:
            if self.bagels[i]:
                self._from = i
                self.itemconfigure(self.bagels[i][-1][1], width=5)
        else:
            self._to = i
            self.move_bagel()

    def move_bagel(self):
        # print(f'Try to move from {self._from} to {self._to}.  ', end='')
        ba = self.bagels
        to = self._to
        fr = self._from
        if fr == to:
            self.itemconfigure(ba[fr][-1][1], width=1)
        elif ba[fr]:
            if not ba[to] or ba[fr][-1][0] < ba[to][-1][0]:
                # moving bagel to respective position
                # print('-->  Ok')
                self.itemconfigure(ba[fr][-1][1], width=1)
                self.coords(ba[fr][-1][1], *self.bagel_coords(to, len(ba[to]), ba[fr][-1][0]))
                self.update()
                ab = ba[fr].pop()
                ab[2] = to
                ba[to].append(ab)
            else:
                # print('-->  Bagel is bigger')
                self.itemconfigure(ba[fr][-1][1], width=1)
                self.itemconfigure(self.text_info, text='Bagel is bigger', fill='red')
                self.update()
                sleep(1)
                self.itemconfigure(self.text_info, text='')
                self.update()
        else:
            # print('-->  There is no bagels')
            self.itemconfigure(self.text_info, text='There is no bagels', fill='red')
            self.update()
            sleep(1)
            self.itemconfigure(self.text_info, text='')
            self.update()
        self.winner()
        self._from = self._to = None

    def winner(self):
        bagels = self.bagels
        i = self.end_position
        if bagels[i] and not bagels[(i + 1) % 3] and not bagels[(i + 2) % 3]:
            # print('You win!!!')
            self.itemconfigure(self.text_info, text='YOU WIN!!!', fill='green')
            self.update()

    def draw_end_position(self, event):
        self.choose_button.configure(command=self.gaming)
        self.end_position = event.x // (self.width // 3)
        # print(self.end_position)
        xc = self.width * (1 / 6 + self.end_position / 3)
        yc = self.height / 2
        w = self.width / 6.1
        h = self.height * 0.4
        if self.end_position_rect is None:
            self.end_position_rect = self.create_rectangle(xc-w, yc-h, xc+w, yc+h, outline='green', width=3)
        else:
            self.coords(self.end_position_rect, xc-w, yc-h, xc+w, yc+h)
        self.update()

    def draw_bagel(self, bagel_column, column_amount, bagel_size):
        coords = self.bagel_coords(bagel_column, column_amount, bagel_size)
        return self.create_rectangle(*coords,
                                     fill=self.colors[bagel_size - 1],
                                     outline='black')

    def bagel_coords(self, bagel_column, column_amount, bagel_size):
        bagel_height = self.height * 0.7 / 10
        bagel_radius = self.width / 30 + bagel_size * self.width / 80
        down_position = self.height * 0.85 - column_amount * bagel_height
        bagel_x_position = self.width * (bagel_column / 3 + 1 / 6)
        return (bagel_x_position - bagel_radius,
                down_position - bagel_height,
                bagel_x_position + bagel_radius,
                down_position)


def main():
    win = Tk()
    win.update()
    width = win.winfo_screenwidth() // 10
    height = win.winfo_screenheight() // 10
    win.geometry(f'{width * 8}x{height * 8}+{width}+{height}')
    HanoiTower()
    win.mainloop()


if __name__ == '__main__':
    main()



import pygame
import random


class Snake:
    def __init__(self):
        self.dis_width = 600
        self.dis_height = 400
        self.snake_speed = 1
        self.speed_adjust = False
        self.snake_block = 25
        self.snake_border = round(self.snake_block / 20)
        self.size_of_food = 20
        self.border = round(30 / self.snake_block) * self.snake_block

        self.white = (255, 255, 255)
        self.yellow = 'yellow'
        self.black = (0, 0, 0)
        self.red = (255, 0, 0)
        self.green = (0, 255, 0)
        self.blue = (0, 0, 255)
        pygame.init()
        self.dis = pygame.display.set_mode((self.dis_width, self.dis_height))
        pygame.display.set_caption('Змійка by VsKesha')
        self.clock = pygame.time.Clock()

    def get_food(self, snake):
        x = round(random.randrange(self.border, self.dis_width - self.snake_block - self.border) / self.snake_block) * self.snake_block
        y = round(random.randrange(self.border, self.dis_height - self.snake_block - self.border) / self.snake_block) * self.snake_block
        for x1, y1 in snake:
            if x == x1 and y == y1:
                x, y = self.get_food(snake)
        return [x, y]

    def your_score(self, score=0):
        score_font = pygame.font.SysFont("comicsansms", 35)
        value = score_font.render("Рахунок: " + str(score) + "    P - пауза", True, self.yellow)
        self.dis.blit(value, [0, 0])
        pygame.display.update()

    def your_speed(self, speed=1):
        speed_font = pygame.font.SysFont("comicsansms", 35)
        value = speed_font.render("Швидкість: " + str(speed), True, self.yellow)
        self.dis.blit(value, [self.dis_width - 180, 0])
        pygame.display.update()

    def show_message(self, msg, color, coords=None, font_size=30):
        coords = coords or (self.dis_width / 7, self.dis_height / 3)
        font_style = pygame.font.SysFont("comicsansms", font_size)
        mesg = font_style.render(msg, True, color)  # .set_alpha(100)
        self.dis.blit(mesg, coords)
        pygame.display.update()

    def refresh_screen(self):
        # makes grid on the screen
        self.dis.fill('black')
        for i in range(self.border + self.snake_block, self.dis_width - self.snake_block, self.snake_block):
            pygame.draw.line(self.dis, 'grey10', [i, self.border], [i, self.dis_height - self.border], self.snake_border * 2)
        for i in range(self.border + self.snake_block, self.dis_height - self.snake_block, self.snake_block):
            pygame.draw.line(self.dis, 'grey10', [self.border, i], [self.dis_width - self.border, i], self.snake_border * 2)
        border_points = [
            [self.border, self.border],
            [self.dis_width - self.border - self.snake_border, self.border],
            [self.dis_width - self.border - self.snake_border, self.dis_height - self.border - self.snake_border],
            [self.border, self.dis_height - self.border - self.snake_border]]
        pygame.draw.lines(self.dis, (150, 150, 150), True, border_points, self.snake_border * 2)

    def draw_food(self, foodx, foody):
        pygame.draw.circle(self.dis, 'yellow', [foodx + round(self.snake_block / 2), foody + round(self.snake_block / 2)],
                           self.size_of_food / 2)
        pygame.draw.circle(self.dis, self.blue, [foodx + round(self.snake_block / 2), foody + round(self.snake_block / 2)], self.size_of_food / 3)

    def draw_snake(self, snake, direct):
        # draw tail
        if len(snake) > 1:
            tail = snake[0]
            previos = snake[1]
            t1 = []
            t2 = []
            t3 = []
            if tail[0] > previos[0]:  # tail goes left
                t1 = [tail[0] + 1, tail[1] + 1]
                t2 = [tail[0] + 1, tail[1] + self.snake_block - 1]
                t3 = [tail[0] + self.snake_block, tail[1] + round(self.snake_block / 2)]
            if tail[0] < previos[0]:  # tail goes right
                t1 = [tail[0] + self.snake_block - 1, tail[1] + 1]
                t2 = [tail[0] + self.snake_block - 1, tail[1] + self.snake_block - 1]
                t3 = [tail[0], tail[1] + round(self.snake_block / 2)]
            else:
                if tail[1] > previos[1]:  # tail goes up
                    t1 = [tail[0] + 1, tail[1] + 1]
                    t2 = [tail[0] + self.snake_block - 1, tail[1] + 1]
                    t3 = [tail[0] + round(self.snake_block / 2), tail[1] + self.snake_block]
                elif tail[1] < previos[1]:  # tail goes down
                    t1 = [tail[0] + 1, tail[1] + self.snake_block - 1]
                    t2 = [tail[0] + self.snake_block - 1, tail[1] + self.snake_block - 1]
                    t3 = [tail[0] + round(self.snake_block / 2), tail[1]]
            points = [t1, t2, t3]
            pygame.draw.polygon(self.dis, self.green, points)
            # pygame.draw.polygon

        # draw body of snake
        for x in snake[1:-1]:
            pygame.draw.rect(self.dis, self.green, [x[0] + 1, x[1] + 1, self.snake_block - 2, self.snake_block - 2])

        # draw_head(snake)
        head = snake[-1]
        center = [head[0] + round(self.snake_block / 2), head[1] + round(self.snake_block / 2)]
        head_radius = self.snake_block / 1.4
        pygame.draw.circle(self.dis, self.green, [center[0], center[1]], head_radius)

        # draw_eyes and tongue
        eyes_color = 'white'
        tongue_color = 'red'
        eye_size = head_radius * 0.35
        distance_forward = round(head_radius * 0.4)
        distance_between_eyes = round(head_radius * 0.5)
        tongue_length = round(head_radius * 0.3)
        tongue_thickness = round(head_radius * 0.05)
        eye_coord1 = []
        eye_coord2 = []
        tongue_coord = []
        if not direct:
            direct = 'right'
        if direct == 'up':
            eye_coord1 = [center[0] - distance_between_eyes, center[1] - distance_forward]
            eye_coord2 = [center[0] + distance_between_eyes, center[1] - distance_forward]
            tongue_coord = [center[0] - tongue_thickness, center[1] - head_radius - tongue_length, tongue_thickness * 2,
                            tongue_length]
        elif direct == 'down':
            eye_coord1 = [center[0] - distance_between_eyes, center[1] + distance_forward]
            eye_coord2 = [center[0] + distance_between_eyes, center[1] + distance_forward]
            tongue_coord = [center[0] - tongue_thickness, center[1] + head_radius, tongue_thickness * 2, tongue_length]
        elif direct == 'left':
            eye_coord1 = [center[0] - distance_forward, center[1] + distance_between_eyes]
            eye_coord2 = [center[0] - distance_forward, center[1] - distance_between_eyes]
            tongue_coord = [center[0] - head_radius - tongue_length, center[1] - tongue_thickness, tongue_length,
                            tongue_thickness * 2]
        elif direct == 'right':
            eye_coord1 = [center[0] + distance_forward, center[1] - distance_between_eyes]
            eye_coord2 = [center[0] + distance_forward, center[1] + distance_between_eyes]
            tongue_coord = [center[0] + head_radius, center[1] - tongue_thickness, tongue_length, tongue_thickness * 2]
        pygame.draw.circle(self.dis, eyes_color, eye_coord1, eye_size)
        pygame.draw.circle(self.dis, 'black', eye_coord1, eye_size / 2)
        pygame.draw.circle(self.dis, eyes_color, eye_coord2, eye_size)
        pygame.draw.circle(self.dis, 'black', eye_coord2, eye_size / 2)
        pygame.draw.rect(self.dis, tongue_color, tongue_coord)
        # pygame.draw.ellipse

    def game_menu(self):
        global snake_speed
        global speed_adjust
        self.refresh_screen()
        self.your_score()
        self.your_speed()
        menu_bg = pygame.Surface([self.dis_width, self.dis_height])  # .set_alpha(50)
        pygame.draw.rect(menu_bg, 'grey20', [0, 0, self.dis_width, self.dis_height])
        menu_bg.set_alpha(200)
        self.dis.blit(menu_bg, [0, 0])
        self.show_message("ЗМІЙКА", "grey5", [self.dis_width / 2 - 100, self.border], 80)
        self.show_message("by vskesha", "dark green", [self.dis_width / 2 + 100, self.border + 50], 30)
        x = self.dis_width / 2 - 200
        y = self.dis_height / 2
        self.show_message("Вибери швидкість:", 'black', [x, y], 50)
        self.show_message('"1" - "9" вибрати швидкість', 'black', [x, y + 50], 30)
        # self.show_message('"A" - автоматично', 'black', [x, y + 80], 30)

        while True:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.game_close()
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_q:
                        self.game_close()
                    elif event.key == pygame.K_a:
                        speed_adjust = True
                        break
                    elif event.key == pygame.K_1:
                        snake_speed = 1
                        break
                    elif event.key == pygame.K_2:
                        snake_speed = 2
                        break
                    elif event.key == pygame.K_3:
                        snake_speed = 3
                        break
                    elif event.key == pygame.K_4:
                        snake_speed = 4
                        break
                    elif event.key == pygame.K_5:
                        snake_speed = 5
                        break
                    elif event.key == pygame.K_6:
                        snake_speed = 6
                        break
                    elif event.key == pygame.K_7:
                        snake_speed = 7
                        break
                    elif event.key == pygame.K_8:
                        snake_speed = 8
                        break
                    elif event.key == pygame.K_9:
                        snake_speed = 9
                        break
            else:
                continue
            break

    def game_pause(self):
        # print("GAME PAUSED")
        paused = True
        x = self.dis_width / 4
        y = self.dis_height / 3
        self.show_message("ПАУЗА!", self.red, [x, y], 50)
        self.show_message("Нажми С для продовження", self.red, [x, y + 65], 35)
        self.show_message("Нажми Q щоб вийти", self.red, [x, y + 100], 35)

        while paused:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.game_close()
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_c:
                        # print("GAME RESUMED")
                        paused = False
                    if event.key == pygame.K_q:
                        self.game_close()

    def game_close(self):
        pygame.quit()
        quit()

    def game_over(self):
        while True:
            x = self.dis_width / 4
            y = self.dis_height / 3
            self.show_message("ПОРАЗКА!", self.red, [x, y], 50)
            self.show_message("Нажми С - нова гра", self.red, [x, y + 65], 35)
            self.show_message("Нажми Q - вийти", self.red, [x, y + 100], 35)

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.game_close()
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_q:
                        self.game_close()
                    if event.key == pygame.K_c:
                        self.gameloop()

    def gameloop(self):
        # getting start position of the snake
        x1 = round(self.dis_width / 2 / self.snake_block) * self.snake_block
        y1 = round(self.dis_height / 2 / self.snake_block) * self.snake_block
        direction = 'right'
        snake_List = []
        Length_of_snake = 3
        Score = 0
        foodx, foody = self.get_food(snake_List)
        global snake_speed
        global speed_adjust
        self.game_menu()

        while True:
            for event in pygame.event.get():
                # print(event)
                if event.type == pygame.QUIT:
                    self.game_close()
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_p:
                        self.game_pause()
                    elif event.key == pygame.K_q:
                        self.game_close()
                    elif event.key == pygame.K_LEFT and direction != 'right':
                        direction = 'left'
                    elif event.key == pygame.K_RIGHT and direction != 'left':
                        direction = 'right'
                    elif event.key == pygame.K_UP and direction != 'down':
                        direction = 'up'
                    elif event.key == pygame.K_DOWN and direction != 'up':
                        direction = 'down'

            if direction == 'left':
                x1 -= self.snake_block
            elif direction == 'right':
                x1 += self.snake_block
            elif direction == 'up':
                y1 -= self.snake_block
            elif direction == 'down':
                y1 += self.snake_block

            if x1 >= self.dis_width - self.border or x1 < self.border or y1 >= self.dis_height - self.border or y1 < self.border:
                self.game_over()

            snake_head = [x1, y1]
            snake_List.append(snake_head)
            if len(snake_List) > Length_of_snake:
                del snake_List[0]

            for x in snake_List[:-1]:
                if x == snake_head:
                    self.game_over()

            self.refresh_screen()
            self.draw_food(foodx, foody)
            self.draw_snake(snake_List, direction)
            self.your_score(Score)
            self.your_speed(snake_speed)

            if x1 == foodx and y1 == foody:
                self.show_message("Нямм", 'yellow', [foodx + self.snake_block, foody + self.snake_block])
                foodx, foody = self.get_food(snake_List)
                Length_of_snake += 1
                Score += snake_speed
            if self.speed_adjust:
                snake_speed = 1 + (Length_of_snake - 3) // 10
            self.clock.tick(snake_speed)


def main():
    snake = Snake()
    snake.gameloop()


if __name__ == '__main__':
    main()



import os
import random
import pygame
from pygame.constants import QUIT, K_DOWN, K_UP, K_LEFT, K_RIGHT
from pathlib import Path


HEIGHT = 700
WIDTH = 1200
COLOR_WHITE = (255, 255, 255)
COLOR_BLACK = (0, 0, 0)
COLOR_BLUE = (0, 0, 255)
COLOR_GREEN = (0, 255, 0)
COLOR_RED = (255, 0, 0)
CURR_DIR = f'{Path(__file__).parent.resolve()}'
PLAYER_IMAGES = [f'{CURR_DIR}/1-{i}.png' for i in range(1, 6)]


def create_enemy():
    enemy = pygame.image.load(f'{CURR_DIR}/enemy.png').convert_alpha()
    enemy_size = (enemy.get_width(), enemy.get_height())
    enemy_rect = pygame.Rect(WIDTH, random.randint(enemy_size[1], HEIGHT-2*enemy_size[1]), *enemy_size)
    enemy_move = [random.randint(-8, -4), 0]
    return [enemy, enemy_rect, enemy_move]


def create_bonus():
    bonus = pygame.image.load(f'{CURR_DIR}/bonus.png').convert_alpha()
    bonus_size = (bonus.get_width(), bonus.get_height())
    bonus_rect = pygame.Rect(random.randint(bonus_size[0], WIDTH-2*bonus_size[0]), -bonus_size[1], *bonus_size)
    bonus_move = [0, random.randint(4, 8)]
    return [bonus, bonus_rect, bonus_move]


def main():
    pygame.init()
    FONT = pygame.font.SysFont('Verdana', 25)
    GAME_OVER_FONT = pygame.font.SysFont('Verdana_bold', 200)
    main_display = pygame.display.set_mode((WIDTH, HEIGHT))
    bg_image = pygame.image.load(f'{CURR_DIR}/background.png')
    bg = pygame.transform.scale(bg_image, (WIDTH, HEIGHT))
    bg_x1, bg_x2 = 0, bg.get_width()
    bg_move = 3

    player = pygame.image.load(f'{CURR_DIR}/player.png').convert_alpha()
    player_size = (player.get_width(), player.get_height())
    player_rect = pygame.Rect(0, (HEIGHT - player_size[1]) // 2, *player_size)
    player_move_down = [0, 5]
    player_move_right = [5, 0]
    player_move_up = [0, -5]
    player_move_left = [-5, 0]

    CREATE_ENEMY = pygame.USEREVENT + 1
    pygame.time.set_timer(CREATE_ENEMY, 2500)
    CREATE_BONUS = pygame.USEREVENT + 2
    pygame.time.set_timer(CREATE_BONUS, 4000)
    CHANGE_IMAGE = pygame.USEREVENT + 3
    pygame.time.set_timer(CHANGE_IMAGE, 150)

    enemies = []
    bonuses = []
    playing = True
    quit_game = False
    score = 0
    image_index = 0

    while playing:
        FPS = pygame.time.Clock()
        FPS.tick(150)

        for event in pygame.event.get():
            if event.type == QUIT:
                playing = False
                quit_game = True
            if event.type == CREATE_ENEMY:
                enemies.append(create_enemy())
            if event.type == CREATE_BONUS:
                bonuses.append(create_bonus())
            if event.type == CHANGE_IMAGE:
                player = pygame.image.load(os.path.join(PLAYER_IMAGES[image_index]))
                image_index += 1
                if image_index >= len(PLAYER_IMAGES):
                    image_index = 0

        bg_x1 -= bg_move
        bg_x2 -= bg_move
        if bg_x1 < -bg.get_width():
            bg_x1, bg_x2 = bg_x2, bg_x2 + bg.get_width()
        main_display.blit(bg, (bg_x1, 0))
        main_display.blit(bg, (bg_x2, 0))
        main_display.blit(player, player_rect)

        keys = pygame.key.get_pressed()

        if keys[K_DOWN] and player_rect.bottom < HEIGHT:
            player_rect = player_rect.move(player_move_down)

        if keys[K_RIGHT] and player_rect.right < WIDTH:
            player_rect = player_rect.move(player_move_right)

        if keys[K_UP] and player_rect.top > 0:
            player_rect = player_rect.move(player_move_up)

        if keys[K_LEFT] and player_rect.left > 0:
            player_rect = player_rect.move(player_move_left)

        for enemy in enemies:
            enemy[1] = enemy[1].move(enemy[2])
            if player_rect.colliderect(enemy[1]):
                playing = False
                explosion = pygame.image.load(f'{CURR_DIR}/explosion.png').convert_alpha()
                enemy[1] = enemy[1].move(-150,-150)
                main_display.blit(explosion, enemy[1])
            else:
                main_display.blit(enemy[0], enemy[1])

        for bonus in bonuses:
            bonus[1] = bonus[1].move(bonus[2])
            main_display.blit(bonus[0], bonus[1])
            if player_rect.colliderect(bonus[1]):
                score += 1
                bonuses.pop(bonuses.index(bonus))
        main_display.blit(FONT.render(f'Рахунок: {score}', True, COLOR_BLACK), (WIDTH-150, 20))
        pygame.display.flip()

        for enemy in enemies:
            if enemy[1].right < 0:
                enemies.pop(enemies.index(enemy))

        for bonus in bonuses:
            if bonus[1].top > HEIGHT:
                bonuses.pop(bonuses.index(bonus))

    while not quit_game:
        for event in pygame.event.get():
            if event.type == QUIT:
                quit_game = True
        main_display.blit(GAME_OVER_FONT.render('GAME OVER', True, COLOR_RED), (200, 300))
        pygame.display.flip()


if __name__ == '__main__':
    main()



import tkinter
import random
import time


class TicTacToe(tkinter.Canvas):
    def __init__(self, window, width=300):
        self.width = width
        super().__init__(window, width=self.width, height=self.width, background='grey70')
        self.state = [None, None, None, None, None, None, None, None, None]
        self.empty_squares = [0, 1, 2, 3, 4, 5, 6, 7, 8]
        self.win_combinations = ((0, 1, 2), (3, 4, 5), (6, 7, 8), (0, 3, 6), (1, 4, 7), (2, 5, 8), (0, 4, 8), (2, 4, 6))
        self.colors = (('brown1', 'brown2', 'coral', 'coral3', 'OliveDrab1', 'orange red'),
                       ('cyan', 'cyan3', 'cornflower blue', 'DarkBlue', 'dark cyan', 'navy'))
        self.get_colors()
        self.bind('<Button-1>', self.click)
        self.game_menu = True
        self.game_running = True
        self.ready = True
        self.game_mode = ''
        self.first_move_x = 1  # 1 - first move x; -1 - first move o
        self.last_win = ''
        self.win_table = {'x_win': 0, 'o_win': 0, 'draw': 0}

    def click(self, event):
        if self.game_menu == True:
            w = self.width / 3
            h = self.width / 10
            n = self.width / 4
            if event.x > 2 * n - w and event.x < 2 * n + w:
                if event.y > n - h and event.y < n + h:
                    self.game_mode = 'comp hard'
                elif event.y > 2 * n - h and event.y < 2 * n + h:
                    self.game_mode = 'comp easy'
                elif event.y > 3 * n - h and event.y < 3 * n + h:
                    self.game_mode = 'person person'
            if self.game_mode:
                self.first_move_x = 1
                self.last_win = ''
                self.win_table = {'x_win': 0, 'o_win': 0, 'draw': 0}
                self.start_new_game()
                self.game_menu = False
        elif self.game_running and self.ready:
            x = self.width / 2
            y = self.width * 30 / 31
            h = self.width / 30
            w = h * 3
            if x - w < event.x < x + w and y - h < event.y < y + h:
                self.game_menu = True
                self.draw_menu()
            else:
                column = event.x // round(self.width / 3)
                row = event.y // round(self.width / 3)
                i = row * 3 + column
                if self.state[i] is None:
                    if self.game_mode == 'person person':
                        if self.first_move_x == 1:
                            if len(self.empty_squares) % 2:
                                self.state[i] = 'x'
                            else:
                                self.state[i] = 'o'
                        if self.first_move_x == -1:
                            if len(self.empty_squares) % 2:
                                self.state[i] = 'o'
                            else:
                                self.state[i] = 'x'
                    else:
                        self.state[i] = 'x'
                    self.empty_squares.remove(i)
                    self.add_figure(column, row)
                    self.get_winner()
                    if not self.last_win and self.game_mode != 'person person':
                        self.bot_move()
        elif self.ready:
            self.start_new_game()

    def bot_move(self):
        if self.empty_squares:
            i = self.best_move()
        else:
            return
        self.state[i] = 'o'
        self.empty_squares.remove(i)
        column = i % 3
        row = i // 3
        self.add_figure(column, row)
        self.get_winner()

    def best_move(self):
        if self.game_mode == 'comp easy' and len(self.empty_squares) > 7:
            return random.choice(self.empty_squares)
        for combination in self.win_combinations:
            i, j, k = combination
            line = [self.state[i], self.state[j], self.state[k]]
            if line.count('o') == 2 and None in line:
                return combination[line.index(None)]
        for combination in self.win_combinations:
            i, j, k = combination
            line = [self.state[i], self.state[j], self.state[k]]
            if line.count('x') == 2 and None in line:
                return combination[line.index(None)]
        free_combination = []
        future_combination = []
        future_xcombination = []
        for combination in self.win_combinations:
            i, j, k = combination
            line = [self.state[i], self.state[j], self.state[k]]
            if line.count('x') == 0:
                for l in range(3):
                    if line[l] != 'o':
                        free_combination.append(combination[l])
                if line.count('o') == 1:
                    for l in range(3):
                        if line[l] != 'o':
                            future_combination.append(combination[l])
            if line.count('o') == 0 and line.count('x') == 1:
                for l in range(3):
                    if line[l] != 'x':
                        future_xcombination.append(combination[l])
        for i in range(9):
            if future_xcombination.count(i) == 2:
                return i
        for i in range(9):
            if future_combination.count(i) == 2:
                return i
        max_count = 0
        max_index = random.choice(self.empty_squares)
        for i in range(9):
            if max_count < free_combination.count(i):
                max_count = free_combination.count(i)
                max_index = i
        return max_index

    def get_colors(self):
        self.x_color = random.choice(self.colors[0])
        self.o_color = random.choice(self.colors[1])

    def draw_menu(self):
        self.delete('all')
        self.draw_button(self.width / 2, self.width / 4, 'КОМП ТЯЖКО')
        self.draw_button(self.width / 2, self.width / 2, 'КОМП ЛЕГКО')
        self.draw_button(self.width / 2, self.width * 3 / 4, 'ДВА ГРАВЦІ')
        self.sign()

    def draw_button(self, x, y, text):
        w = self.width / 3
        h = self.width / 10
        self.create_oval(x - w - h, y - h, x - w + h, y + h, fill='grey15', outline='black')
        self.create_oval(x + w - h, y - h, x + w + h, y + h, fill='grey15', outline='black')
        self.create_rectangle(x - w, y - h, x + w, y + h, fill='grey15', outline='grey15')
        self.create_line(x - w, y - h, x + w, y - h, fill='black')
        self.create_line(x - w, y + h, x + w, y + h, fill='black')
        self.create_text(x, y + h / 5, text=text, font=('Courier', round(h * 0.75), 'bold'), fill='white')

    def add_figure(self, column, row):
        if len(self.empty_squares) % 2:
            self.add_o(column, row)
        else:
            self.add_x(column, row)

    def add_x(self, column, row):
        x = column * round(self.width / 3)
        y = row * round(self.width / 3)
        self.create_line(
            x + round(self.width / 20),
            y + round(self.width / 20),
            x + round(self.width * 85 / 300),
            y + round(self.width * 85 / 300),
            width=round(self.width / 50),
            fill=self.x_color)
        self.create_line(
            x + round(self.width * 85 / 300),
            y + round(self.width / 20),
            x + round(self.width / 20),
            y + round(self.width * 85 / 300),
            width=round(self.width / 50),
            fill=self.x_color)
        self.update()

    def add_o(self, column, row):
        x = column * round(self.width / 3)
        y = row * round(self.width / 3)
        self.create_oval(
            x + round(self.width / 20),
            y + round(self.width / 20),
            x + round(self.width * 85 / 300),
            y + round(self.width * 85 / 300),
            width=round(self.width / 50),
            outline=self.o_color)
        self.update()

    def draw_lines(self):
        self.delete('all')
        x = round(self.width / 3)
        s = self.width
        self.create_line(x, 0, x, s, fill='grey20')
        self.create_line(2 * x, 0, 2 * x, s, fill='grey20')
        self.create_line(0, x, s, x, fill='grey20')
        self.create_line(0, 2 * x, s, 2 * x, fill='grey20')
        self.update()
        self.sign()

    def draw_explanation(self):
        pos_x = self.width / 6
        pos_o = self.width / 2
        width = self.width / 100
        size = self.width / 35
        if self.game_mode == 'person person':
            if self.first_move_x == 1:
                text = ' - Гравець1      - Гравець2 '
            else:
                text = ' - Гравець2      - Гравець1 '
        else:
            if self.first_move_x == 1:
                text = ' - Гравець       - Компʼютер'
            else:
                text = ' - Компʼютер     - Гравець  '
        self.create_line(pos_x, width, pos_x + size, width + size, width=width, fill=self.x_color)
        self.create_line(pos_x + size, width, pos_x, width + size, width=width, fill=self.x_color)
        self.create_oval(pos_o, width, pos_o + size, width + size, width=width, outline=self.o_color)
        self.create_text(pos_o, size, text=text, font=('Courier', round(size), 'bold'), fill='grey15')
        self.update()

    def draw_menu_button(self):
        x = self.width / 2
        y = self.width * 30 / 31
        h = self.width / 30
        w = h * 3
        self.create_rectangle(x - w, y - h, x + w, y + h, outline='black', fill='grey15')
        self.create_text(x, y + h / 3, text='МЕНЮ', font=('Courier', round(h * 1.5), 'bold'), fill='white')
        self.update()

    def draw_winline(self):
        if self.win_line < 3:  # draw horizontal winline
            self.create_line(
                0,
                self.width * (self.win_combinations[self.win_line][0] / 9 + 1 / 6),
                self.width,
                self.width * (self.win_combinations[self.win_line][0] / 9 + 1 / 6),
                width=round(self.width / 50),
                fill=random.choice(random.choice(self.colors)))
        elif self.win_line < 6:  # draw vertical winline
            self.create_line(
                self.width * (self.win_combinations[self.win_line][0] / 3 + 1 / 6),
                0,
                self.width * (self.win_combinations[self.win_line][0] / 3 + 1 / 6),
                self.width,
                width=round(self.width / 50),
                fill=random.choice(random.choice(self.colors)))
        elif self.win_line == 6:  # first diagonal line
            self.create_line(
                0,
                0,
                self.width,
                self.width,
                width=round(self.width / 50),
                fill=random.choice(random.choice(self.colors)))
        elif self.win_line == 7:  # second diagonal line
            self.create_line(
                self.width,
                0,
                0,
                self.width,
                width=round(self.width / 50),
                fill=random.choice(random.choice(self.colors)))
        self.update()

    def draw_wintable(self):
        text = ''
        if self.last_win == 'x_win':
            if self.game_mode == 'person person':
                text = '1-й виграв'
            else:
                text = 'Ви виграли'
        elif self.last_win == 'o_win':
            if self.game_mode == 'person person':
                text = '2-й виграв'
            else:
                text = 'Ви програли'
        else:
            text = 'Нічия'
        self.create_text(self.width / 2, self.width / 2,
                         text=text, font=('Arial', round(self.width / 10), 'bold'), fill='grey15')
        self.update()
        time.sleep(1)
        self.delete('all')
        self.sign()
        if self.game_mode == 'person person':
            text = ['Гравець1', 'Гравець2']
        else:
            text = ['Перемоги', 'Поразки ']
        self.create_text(self.width / 2, self.width / 2.5,
                         text=f'{text[0]} - {self.win_table["x_win"]}\n{text[1]} - {self.win_table["o_win"]}\nНічия    - {self.win_table["draw"]}',
                         font=('Courier', round(self.width / 11), 'bold'), fill='grey15')
        self.update()
        time.sleep(1)
        self.create_text(self.width / 2, self.width * 3 / 4,
                         text='Клацни для продовження',
                         font=('Arial', round(self.width / 25), 'italic'), fill='brown')

    def sign(self):
        self.create_text(self.width * 9 / 10, self.width * 39 / 40,
                         text='by VsKesha', font=('Arial', round(self.width / 40), 'italic'), fill='Blue')
        self.update()

    def start_new_game(self):
        self.state = [None, None, None, None, None, None, None, None, None]
        self.empty_squares = [0, 1, 2, 3, 4, 5, 6, 7, 8]
        self.get_colors()
        self.delete('all')
        self.draw_lines()
        self.draw_explanation()
        self.draw_menu_button()
        self.game_running = True
        if self.first_move_x == -1 and self.game_mode != 'person person':
            self.bot_move()

    def get_winner(self):
        self.last_win = ''
        if not self.empty_squares:
            self.last_win = 'draw'
            self.win_line = 9
        for n, combination in enumerate(self.win_combinations):
            i, j, k = combination
            if self.state[i] and self.state[i] == self.state[j] and self.state[j] == self.state[k]:
                self.win_line = n
                if self.state[i] == 'x':
                    self.last_win = 'x_win'
                elif self.state[i] == 'o':
                    self.last_win = 'o_win'
        if self.last_win:
            self.game_ending()

    def game_ending(self):
        self.ready = False
        self.game_running = False
        self.win_table[self.last_win] += 1
        if self.win_line < 9:
            for i in range(5):
                self.draw_winline()
                time.sleep(0.3)
        self.draw_wintable()
        if self.last_win == 'x_win':
            self.first_move_x = 1
        elif self.last_win == 'o_win':
            self.first_move_x = -1
        else:
            self.first_move_x *= -1
        self.ready = True


def main():
    window = tkinter.Tk()
    window.title('Хрестики-Нyлики')
    game = TicTacToe(window, 600)
    game.pack()
    game.draw_menu()
    window.mainloop()


if __name__ == '__main__':
    main()
